﻿using System;
using System.IO;

namespace Gra
{
    public class Gra
    {
        private readonly Plansza _plansza;
        private readonly Plansza _planszaNastępna;
        public char Znak { get; set; }
        public ConsoleColor KolorTła { get; set; }
        public ConsoleColor KolorZnaków { get; set; }

        public int NumerCyklu {
            get { return _numerCyklu; }
        }

        private int _liczbaZywych;
        private int _numerCyklu;


        public Gra(Plansza plansza)
        {
            _plansza = plansza;
            _planszaNastępna = new Plansza(_plansza.Szerokość, _plansza.Wysokość);
            Znak = '*';
            KolorZnaków = ConsoleColor.White;
            KolorTła = ConsoleColor.DarkGreen;
        }

        public Gra()
            : this(new Plansza())
        { }

        public void Rysuj()
        {
            Console.Clear();

            Console.Title = "Gra Życie";

            Console.ResetColor();
            Console.SetCursorPosition(0, 0);
            Console.WriteLine("Liczba żywych      : {0} / {1} [{2:F1}%]             ", _liczbaZywych, _plansza.Szerokość * _plansza.Wysokość,
                100.0 * _liczbaZywych / (_plansza.Szerokość * _plansza.Wysokość));
            Console.WriteLine("Cykl               : {0}                             ", NumerCyklu);
            Console.WriteLine("Wymiary            : {0}x{1}                         ", _plansza.Szerokość, _plansza.Wysokość);

            const int startx = 3;
            const int starty = 5;

            Console.ForegroundColor = KolorZnaków;
            Console.BackgroundColor = KolorTła;

            for (int i = 0; i < _plansza.Szerokość; i++)
                for (int j = 0; j < _plansza.Wysokość; j++)
                {
                    Console.SetCursorPosition(startx + i, starty + j);
                    if (_plansza.DajKomórkę(i, j).CzyŻywa) Console.Write(Znak);
                    else Console.Write(' ');
                }

            Console.ResetColor();
            Console.WriteLine("\n");
            Console.WriteLine("Wciśnij Spację aby zatrzymać.");
        }

        public void Cykkl()
        {
            for (int i = 0; i < _plansza.Szerokość; i++)
            {
                for (int j = 0; j < _plansza.Wysokość; j++)
                {
                    int x = _plansza.IleSąsiadów(i, j);
                    
                    if (_plansza.DajKomórkę(i,j).CzyŻywa == false  && x == 3)
                        _planszaNastępna.DajKomórkę(i, j).CzyŻywa = true;
        
                    if (_plansza.DajKomórkę(i,j).CzyŻywa && (x < 2 || x > 3))
                        _planszaNastępna.DajKomórkę(i, j).CzyŻywa = false;
                }
            }

            _liczbaZywych = 0;
            for (int i = 0; i < _plansza.Szerokość; i++)
                for (int j = 0; j < _plansza.Wysokość; j++)
                {
                    bool c = _planszaNastępna.DajKomórkę(i, j).CzyŻywa;
                    
                    _plansza.DajKomórkę(i, j).CzyŻywa = c;
                    if (c)
                    {
                        _plansza.DajKomórkę(i, j).Wiek++;
                        _liczbaZywych++;
                    }
                    else _plansza.DajKomórkę(i, j).Wiek = 0;
                }

            _numerCyklu++;
        }

        public void Inicjij(double p = 0.25)
        {
            Random rand = new Random();
            _liczbaZywych = 0;
            for (int i = 0; i < _plansza.Szerokość; i++)
            {
                for (int j = 0; j < _plansza.Wysokość; j++)
                {
                    if (rand.NextDouble() <= p)
                    {
                        _plansza.DajKomórkę(i, j).CzyŻywa = true;
                        _liczbaZywych++;
                    }
                    else _plansza.DajKomórkę(i, j).CzyŻywa = false;
                }
            }
            _numerCyklu = 0;
        }


        public bool Zapisz(string nazwaPliku)
        {
            using (StreamWriter plik = new StreamWriter(nazwaPliku))
            {
                plik.WriteLine("Liczba żywych      : {0} / {1} [{2:F1}%]             ", _liczbaZywych,  _plansza.Szerokość * _plansza.Wysokość, 100.0 * _liczbaZywych / (_plansza.Szerokość * _plansza.Wysokość));
                plik.WriteLine("Cykl               : {0}                             ", NumerCyklu);
                plik.WriteLine("Wymiary            : {0}x{1}                         ", _plansza.Szerokość, _plansza.Wysokość);

                for (int i = 0; i < _plansza.Wysokość; i++)
                {
                    for (int j = 0; j < _plansza.Szerokość; j++)
                    {
                        if (_plansza.DajKomórkę(i,j).CzyŻywa) plik.Write(Znak);
                        else plik.Write(' ');
                    }
                    plik.WriteLine();
                }
            }
            return true;
        }
    }
}
